searchData={"items":[{"type":"module","title":"Ndim","doc":"The `Ndim` lib provides functions for working with n-dimensional lists.\n\nThis module is particularly useful when you need to:\n* Map functions over deeply nested lists at a specific level while maintaining their shape\n* Transform between the n-dimensional list and the coordinated map","ref":"Ndim.html"},{"type":"module","title":"Terminology - Ndim","doc":"","ref":"Ndim.html#module-terminology"},{"type":"module","title":"N-dimensional List - Ndim","doc":"An n-dimensional list is a regularly nested list structure where n represents the depth\nof nesting. Each level must maintain consistent structure:\n\n# 1-dimensional list (vector)\n\n```elixir\n[1, 2, 3]\n```\n\n# 2-dimensional list (matrix)\n\n```elixir\n[\n  [1, 2, 3],\n  [4, 5, 6]\n]\n```\n\n# 3-dimensional list (cube)\n\n```elixir\n[\n  [\n    [1, 2], [3, 4]\n  ],\n  [\n    [5, 6], [7, 8]\n  ]\n]\n```","ref":"Ndim.html#module-n-dimensional-list"},{"type":"module","title":"Coordinate map - Ndim","doc":"A coordinate map is a map representation of an n-dimensional list where each value\nis keyed by its dimensional coordinates. This is useful for sparse data or when\nyou need direct coordinate access:\n\n#### From 2-dimensional list to 2d coordinate map\n\n```elixir\n[\n  [1, 2],     -->  %{{0, 0} => 1, {0, 1} => 2,\n  [3, 4]             {1, 0} => 3, {1, 1} => 4}\n]\n```\n\n#### From 3-dimensional list to 3d coordinate map\n\n```elixir\n[\n  [[1, 2]],   -->  %{{0, 0, 0} => 1, {0, 0, 1} => 2,\n  [[3, 4]]           {1, 0, 0} => 3, {1, 0, 1} => 4}\n]\n```","ref":"Ndim.html#module-coordinate-map"},{"type":"module","title":"Core functions - Ndim","doc":"* `d2map/2` ... `d5map/2` - Map a function over a 2-dimensonal list (upto 5)\n* `dmap/3` - The general version which maps a function over elements at any specified dimensional list\n* `to_coordinate_map/1` - Transform a n-dimensional list to a coordinate map","ref":"Ndim.html#module-core-functions"},{"type":"module","title":"Examples - Ndim","doc":"Map a function over a 2-dimensional list\n\n  ```elixir\n  iex> numbers = [[1, 2], [3, 4]]\n  iex> Ndim.d2map(numbers, fn x -> x * 2 end)\n  [[2, 4], [6, 8]]\n  ```\n\nTransform a 2-dimensional list to a coordinate map\n\n  ```elixir\n  iex> vector = [[1, 2], [3, 4]]\n  iex> Ndim.to_coordinate_map(vector)\n  %{{0, 0} => 1, {0, 1} => 2, {1, 0} => 3, {1, 1} => 4}\n  ```","ref":"Ndim.html#module-examples"},{"type":"function","title":"Ndim.d2map/2","doc":"Maps a function over a 2-dimensional list.\nTakes a 2-dimensional list as the first argument, followed by a mapping function.\nThis is equivalent to calling `dmap(list, 2, fun)`.","ref":"Ndim.html#d2map/2"},{"type":"function","title":"Examples - Ndim.d2map/2","doc":"```elixir\n   iex> numbers = [[1, 2], [3, 4], [5, 6]]\n   iex> Ndim.d2map(numbers, fn x -> x * 2 end)\n   [[2, 4], [6, 8], [10, 12]]\n\n   iex> strings = [[\"a\", \"b\"], [\"c\", \"d\"]]\n   iex> Ndim.d2map(strings, &String.upcase/1)\n   [[\"A\", \"B\"], [\"C\", \"D\"]]\n   ```","ref":"Ndim.html#d2map/2-examples"},{"type":"function","title":"Ndim.d3map/2","doc":"Maps a function over a 3-dimensional list.\nTakes a 3-dimensional list as the first argument, followed by a mapping function.\nThis is equivalent to calling `dmap(list, 3, fun)`.","ref":"Ndim.html#d3map/2"},{"type":"function","title":"Examples - Ndim.d3map/2","doc":"```elixir\n   iex> numbers = [[[1, 2], [3, 4]], [[5, 6], [7, 8]]]\n   iex> Ndim.d3map(numbers, fn x -> x + 1 end)\n   [[[2, 3], [4, 5]], [[6, 7], [8, 9]]]\n\n   iex> strings = [[[\"hello\", \"world\"], [\"foo\"]], [[\"bar\"], [\"baz\", \"qux\"]]]\n   iex> Ndim.d3map(strings, &String.capitalize/1)\n   [[[\"Hello\", \"World\"], [\"Foo\"]], [[\"Bar\"], [\"Baz\", \"Qux\"]]]\n   ```","ref":"Ndim.html#d3map/2-examples"},{"type":"function","title":"Ndim.d4map/2","doc":"Maps a function over a 4-dimensional list.\nTakes a 4-dimensional list as the first argument, followed by a mapping function.\nThis is equivalent to calling `dmap(list, 4, fun)`.","ref":"Ndim.html#d4map/2"},{"type":"function","title":"Examples - Ndim.d4map/2","doc":"```elixir\n   iex> numbers = [[[[1, 2], [3]], [[4, 5]]], [[[6]], [[7, 8], [9]]]]\n   iex> Ndim.d4map(numbers, fn x -> x * 2 end)\n   [[[[2, 4], [6]], [[8, 10]]], [[[12]], [[14, 16], [18]]]]\n\n   iex> strings = [[[[{\"a\", 1}]]], [[[{\"b\", 2}]]]]\n   iex> Ndim.d4map(strings, fn {str, num} -> {String.upcase(str), num} end)\n   [[[[{\"A\", 1}]]], [[[{\"B\", 2}]]]]\n   ```","ref":"Ndim.html#d4map/2-examples"},{"type":"function","title":"Ndim.d5map/2","doc":"Maps a function over a 5-dimensional list.\nTakes a 5-dimensional list as the first argument, followed by a mapping function.\nThis is equivalent to calling `dmap(list, 5, fun)`.","ref":"Ndim.html#d5map/2"},{"type":"function","title":"Examples - Ndim.d5map/2","doc":"```elixir\n   iex> numbers = [[[[[1]], [[2]]], [[[3]]]], [[[[4]]], [[[5]], [[6]]]]]\n   iex> Ndim.d5map(numbers, fn x -> x + 10 end)\n   [[[[[11]], [[12]]], [[[13]]]], [[[[14]]], [[[15]], [[16]]]]]\n\n   iex> data = [[[[[%{x: 1}]], [[%{x: 2}]]]], [[[[%{x: 3}]]]]]\n   iex> Ndim.d5map(data, fn map -> Map.update!(map, :x, & &1 * 2) end)\n   [[[[[%{x: 2}]], [[%{x: 4}]]]], [[[[%{x: 6}]]]]]\n   ```","ref":"Ndim.html#d5map/2-examples"},{"type":"function","title":"Ndim.dmap/3","doc":"Maps a function over a specified dimensional list.\nTakes a n-dimensional list as the first argument, followed by the nesting level, then a mapping function.","ref":"Ndim.html#dmap/3"},{"type":"function","title":"Parameters - Ndim.dmap/3","doc":"* `list` - A n-dimensional nested list to transform\n  * `dim` - The dimension at which to apply the function (1-based indexing)\n  * `fun` - The function to apply at the specified dimension","ref":"Ndim.html#dmap/3-parameters"},{"type":"function","title":"Examples - Ndim.dmap/3","doc":"```elixir\n  iex> numbers = [[1, 2], [3, 4], [5, 6]]\n  iex> Ndim.dmap(numbers, 2, fn i -> i * 10 end)\n  [[10, 20], [30, 40], [50, 60]]\n\n  iex> nested = [[[\"a\", \"b\"], [\"c\", \"d\"]], [[\"e\", \"f\"], [\"g\", \"h\"]], [[\"i\", \"j\"], [\"k\", \"l\"]]]\n  iex> Ndim.dmap(nested, 3, &String.upcase/1)\n  [[[\"A\", \"B\"], [\"C\", \"D\"]], [[\"E\", \"F\"], [\"G\", \"H\"]], [[\"I\", \"J\"], [\"K\", \"L\"]]]\n  ```","ref":"Ndim.html#dmap/3-examples"},{"type":"function","title":"Ndim.get_depth/1","doc":"Gets the depth (dimension) of a regular nested list. Returns 0 for an empty list,\n1 for a flat list, and n for an n-dimensional nested list. Only works with regular\nnested structures where elements at the same depth share the same dimensionality.","ref":"Ndim.html#get_depth/1"},{"type":"function","title":"Examples - Ndim.get_depth/1","doc":"```elixir\n   iex> Ndim.get_depth([])\n   0\n\n   iex> Ndim.get_depth([1, 2, 3])\n   1\n\n   iex> Ndim.get_depth([[1, 2], [3, 4]])\n   2\n\n   iex> Ndim.get_depth([[[1, 2]], [[3, 4]]])\n   3\n   ```","ref":"Ndim.html#get_depth/1-examples"},{"type":"function","title":"Error cases - Ndim.get_depth/1","doc":"```elixir\n  iex> Ndim.get_depth([1, [2, 3]])\n  ** (ArgumentError) list is not a regular nested structure\n\n  iex> Ndim.get_depth([[1, 2], [3]])\n  ** (ArgumentError) list is not a regular nested structure\n  ```","ref":"Ndim.html#get_depth/1-error-cases"},{"type":"function","title":"Return Value - Ndim.get_depth/1","doc":"Returns an integer representing the depth (dimension) of the nested list.\nFor irregular structures, raises an ArgumentError.\n\nNote: The depth counting starts from 0 for an empty list, 1 for a flat list,\nand increments by one for each level of regular nesting.","ref":"Ndim.html#get_depth/1-return-value"},{"type":"function","title":"Ndim.get_depth/2","doc":"","ref":"Ndim.html#get_depth/2"},{"type":"function","title":"Ndim.regular?/1","doc":"","ref":"Ndim.html#regular?/1"},{"type":"function","title":"Ndim.to_coordinate_map/1","doc":"","ref":"Ndim.html#to_coordinate_map/1"},{"type":"function","title":"Ndim.to_coordinate_map/2","doc":"Converts an n-dimensional nested list into a coordinate map where each value\nis keyed by its dimensional coordinates. The dimension can be specified to\nhandle specific cases, or automatically detected from the list structure.","ref":"Ndim.html#to_coordinate_map/2"},{"type":"function","title":"Parameters - Ndim.to_coordinate_map/2","doc":"* `list` - An n-dimensional nested list to convert\n * `dimension` - (Optional) The expected dimension of the nested list","ref":"Ndim.html#to_coordinate_map/2-parameters"},{"type":"function","title":"Examples - Ndim.to_coordinate_map/2","doc":"```elixir\n   # Automatic dimension detection\n   iex> matrix = [[1, 2], [3, 4]]\n   iex> Ndim.to_coordinate_map(matrix)\n   %{{0, 0} => 1, {0, 1} => 2, {1, 0} => 3, {1, 1} => 4}\n\n   iex> cube = [[[1, 2]], [[3, 4]]]\n   iex> Ndim.to_coordinate_map(cube)\n   %{{0, 0, 0} => 1, {0, 0, 1} => 2, {1, 0, 0} => 3, {1, 0, 1} => 4}\n\n   # Explicitly specifying 2D\n   iex> matrix = [[1, 2], [3, 4]]\n   iex> Ndim.to_coordinate_map(matrix, 2)\n   %{{0, 0} => 1, {0, 1} => 2, {1, 0} => 3, {1, 1} => 4}\n\n   # Explicitly specifying 3D\n   iex> cube = [[[1, 2]], [[3, 4]]]\n   iex> Ndim.to_coordinate_map(cube, 3)\n   %{{0, 0, 0} => 1, {0, 0, 1} => 2, {1, 0, 0} => 3, {1, 0, 1} => 4}\n   ```","ref":"Ndim.html#to_coordinate_map/2-examples"},{"type":"function","title":"Return Value - Ndim.to_coordinate_map/2","doc":"Returns a map where keys are tuples of coordinates and values are the corresponding\nelements from the input list.\n\nNote: Coordinates use zero-based indexing, following programming conventions.","ref":"Ndim.html#to_coordinate_map/2-return-value"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}